
/**
  TMR5 Generated Driver API Source File 

  @Company
    Microchip Technology Inc.

  @File Name
    tmr5.c

  @Summary
    This is the generated source file for the TMR5 driver using PIC24 / dsPIC33 / PIC32MM MCUs

  @Description
    This source file provides APIs for driver for TMR5. 
    Generation Information : 
        Product Revision  :  PIC24 / dsPIC33 / PIC32MM MCUs - pic24-dspic-pic32mm : v1.45
        Device            :  PIC24FJ128GA202
    The generated drivers are tested against the following:
        Compiler          :  XC16 1.32
        MPLAB 	          :  MPLAB X 3.61
*/

/*
    (c) 2016 Microchip Technology Inc. and its subsidiaries. You may use this
    software and any derivatives exclusively with Microchip products.

    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
    WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
    PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION
    WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.

    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
    BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
    FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
    ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
    THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.

    MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE
    TERMS.
*/

/**
  Section: Included Files
*/

#include <xc.h>
#include "tmr5.h"

/**
  Section: Data Type Definitions
*/

/** TMR Driver Hardware Instance Object

  @Summary
    Defines the object required for the maintainence of the hardware instance.

  @Description
    This defines the object required for the maintainence of the hardware
    instance. This object exists once per hardware instance of the peripheral.

  Remarks:
    None.
*/

typedef struct _TMR_OBJ_STRUCT
{
    /* Timer Elapsed */
    bool                                                    timerElapsed;
    /*Software Counter value*/
    uint8_t                                                 count;

} TMR_OBJ;

static TMR_OBJ tmr5_obj;

/**
  Section: Driver Interface
*/


void TMR5_Initialize (void)
{
    //TMR5 0; 
    TMR5 = 0x0000;
    //Period = 0.01 s; Frequency = 2000000 Hz; PR5 20000; 
    PR5 = 0x4E20;
    //TCKPS 1:1; TON enabled; TSIDL disabled; TCS FOSC/2; TECS SOSC; TGATE disabled; 
    T5CON = 0x8000;

    
    IFS1bits.T5IF = false;
    IEC1bits.T5IE = true;
	
    tmr5_obj.timerElapsed = false;

}



void __attribute__ ( ( interrupt, no_auto_psv ) ) _T5Interrupt (  )
{
    /* Check if the Timer Interrupt/Status is set */

    //***User Area Begin

    // ticker function call;
    // ticker is 1 -> Callback function gets called everytime this ISR executes
    TMR5_CallBack();

    //***User Area End

    tmr5_obj.count++;
    tmr5_obj.timerElapsed = true;
    IFS1bits.T5IF = false;
}


void TMR5_Period16BitSet( uint16_t value )
{
    /* Update the counter values */
    PR5 = value;
    /* Reset the status information */
    tmr5_obj.timerElapsed = false;
}

uint16_t TMR5_Period16BitGet( void )
{
    return( PR5 );
}

void TMR5_Counter16BitSet ( uint16_t value )
{
    /* Update the counter values */
    TMR5 = value;
    /* Reset the status information */
    tmr5_obj.timerElapsed = false;
}

uint16_t TMR5_Counter16BitGet( void )
{
    return( TMR5 );
}




unsigned char glob_DirSave = 1;
unsigned char glob_ResetServo = 50; // 10ms Timer Interrupt => 0,5s
unsigned long glob_RotationDelay = 0;
unsigned long glob_OLD_RotationDelay;


extern unsigned char __CN_SEMAPHORE;
extern unsigned char glob_CN_SIGNAL;

unsigned long curOpeningGrade = 40;
unsigned char ResetCloseHit = 0;

void __attribute__ ((weak)) TMR5_CallBack(void) // Servoansteuerung
{
    // Add your custom callback code here
    if(__CN_SEMAPHORE)
        return;
    
    if(glob_CN_SIGNAL)
    {
        // neues Signal von Gabellichtschranke
        
        unsigned long RotationDelayNew = glob_RotationDelay;
        unsigned long RotationDelayOld = glob_OLD_RotationDelay;
        
        if(RotationDelayNew > (RotationDelayOld*2)) // Dann war letzte Umdrehung Bereich ohne Stator => jetzt Stator
        {
            glob_DirSave = 1; // Schiebetr zu
        }
        
        
        glob_OLD_RotationDelay = RotationDelayNew;
        glob_RotationDelay = 0;
        
        
        
        glob_CN_SIGNAL = 0;
        
        if(glob_DirSave) // Schiebetr zu
        {
                OC1R = 4000; // 10% Duty  schnell links gegen uhrzeigersinn
                //glob_ResetServo = 16; // viertel umdrehung gegen uhrzeiger
                glob_ResetServo = (curOpeningGrade*11)/10;  // etwas lnger zufahren als auf
                ResetCloseHit = 1;
        }
        else // Schiebetr auf
        {
                OC1R = 2000;  // 5% Duty schnell rechts
                //glob_ResetServo = 16; // viertel umdrehung im uhrzeiger
                
                if(ResetCloseHit == 2)
                    curOpeningGrade++;
                else if(ResetCloseHit == 1)
                {
                    if(curOpeningGrade>1)
                        curOpeningGrade--;
                }
                
                glob_ResetServo = curOpeningGrade; // viertel umdrehung gegen uhrzeiger
        }
            
        glob_DirSave = !glob_DirSave;
    }
    
    
    
    if(glob_ResetServo>0)
        glob_ResetServo--;
    else
    {
        OC1R = 0;
        if(ResetCloseHit == 1)
            ResetCloseHit = 2;
    }
    
    glob_RotationDelay++;
    
}

void TMR5_Start( void )
{
    /* Reset the status information */
    tmr5_obj.timerElapsed = false;

    /*Enable the interrupt*/
    IEC1bits.T5IE = true;

    /* Start the Timer */
    T5CONbits.TON = 1;
}

void TMR5_Stop( void )
{
    /* Stop the Timer */
    T5CONbits.TON = false;

    /*Disable the interrupt*/
    IEC1bits.T5IE = false;
}

bool TMR5_GetElapsedThenClear(void)
{
    bool status;
    
    status = tmr5_obj.timerElapsed;

    if(status == true)
    {
        tmr5_obj.timerElapsed = false;
    }
    return status;
}

int TMR5_SoftwareCounterGet(void)
{
    return tmr5_obj.count;
}

void TMR5_SoftwareCounterClear(void)
{
    tmr5_obj.count = 0; 
}

/**
 End of File
*/
